@@ -3,13 +3,13 @@
 #------------------------------------------------------------------------------
 # LICENSE:
 # This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free 
+# the terms of the GNU Lesser General Public License as published by the Free
 # Software  Foundation; either version 3 of the License, or (at your option) any
 # later version. See http://www.gnu.org/licenses/lgpl-3.0.txt.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 # details.
 #
 # You should have received a copy of the GNU Lesser General Public License along
@@ -19,9 +19,9 @@
 # CHANGELOG:
 # 2006-07-15 v0.1.1 AN: - released version
 # 2007-10-09 v0.2.0 PL: - fixed error with deprecated string exceptions
-#			- added optional timeout to sockets to avoid blocking 
+#			- added optional timeout to sockets to avoid blocking
 #			  operations
-# 2010-07-11 v0.2.1 AN: - change all raise exception (was deprecated), license 
+# 2010-07-11 v0.2.1 AN: - change all raise exception (was deprecated), license
 #						  change to LGPL
 # 2010-07-12 v0.2.2 TK: - PEP8 compliance
 #						  isolating send and receive functions
@@ -53,6 +53,9 @@
 # 2015-03-14 v0.3.14 AN : - Bug correction for clamd.conf default path
 # 2015-06-04 v0.3.15 AN : - optimization in scan_stream
 # 2015-10-21 v0.3.16 JMS : - avoid EICAR detection in py3 pyc file
+# 2016-08-07 v0.3.17 AN: - typo change (Connexion to Connection)
+# 2017-08-27 v0.4.0 RC: - modified scan_stream() to add support for passing file-like objects
+#                   BM: - add allmatchscan with file and directory support
 #------------------------------------------------------------------------------
 # TODO:
 # - improve tests for Win32 platform (avoid to write EICAR file to disk, or
@@ -71,10 +74,12 @@ pyclamd.py
 
 Author : Alexandre Norman - norman()xael.org
 Contributors :
+ - BM : Brandon Murphy - bitbucket () zoomequipd.com
  - JB :  Joe Brandt  - brandt.joe () gmail.com
  - JMS: Jack Saunders - jack () oldstlabs.com
  - JS : Joni Salonen - joni.salonen () qindel.com
  - PL :  Philippe Lagadec - philippe.lagadec()laposte.net
+ - RC : Robert Coup
  - SK : Scott Kitterman - debian () kitterman.com
  - TK :  Thomas Kastner - tk()underground8.com
  - TKL : Thomas Kluyver - thomas () kluyver.me.uk
@@ -129,7 +134,7 @@ True
 >>> os.remove('/tmp/EICAR-éèô请收藏我们的网址')
 """
 
-__version__ = "0.3.17"
+__version__ = "0.4.0"
 
 
 # $Source$
@@ -169,7 +174,7 @@ class _ClamdGeneric(object):
     """
     Abstract class for clamd
     """
-    
+
     def EICAR(self):
         """
         returns Eicar test string
@@ -184,7 +189,7 @@ class _ClamdGeneric(object):
 
         EICAR = base64.b64decode(eicar_b64.encode('ascii'))
         return EICAR
-        
+
 
     def ping(self):
         """
@@ -213,7 +218,7 @@ class _ClamdGeneric(object):
         return
 
 
-    
+
     def version(self):
         """
         Get Clamscan version
@@ -253,7 +258,7 @@ class _ClamdGeneric(object):
 
         return result
 
-    
+
     def reload(self):
         """
         Force Clamd to reload signature database
@@ -270,14 +275,14 @@ class _ClamdGeneric(object):
             self._send_command('RELOAD')
             result = self._recv_response()
             self._close_socket()
-            
+
         except socket.error:
             raise ConnectionError('Could probably not reload signature database')
 
         return result
 
 
-    
+
     def shutdown(self):
         """
         Force Clamd to shutdown and exit
@@ -296,7 +301,7 @@ class _ClamdGeneric(object):
             raise ConnectionError('Could probably not shutdown clamd')
 
 
-    
+
     def scan_file(self, file):
         """
         Scan a file or directory given by filename and stop on first virus or error found.
@@ -335,7 +340,7 @@ class _ClamdGeneric(object):
                 if status == 'ERROR':
                     dr[filename] = ('ERROR', '{0}'.format(reason))
                     return dr
-                    
+
                 elif status == 'FOUND':
                     dr[filename] = ('FOUND', '{0}'.format(reason))
 
@@ -344,7 +349,7 @@ class _ClamdGeneric(object):
             return None
         return dr
 
-    
+
 
 
 
@@ -385,7 +390,7 @@ class _ClamdGeneric(object):
 
                     if status == 'ERROR':
                         dr[filename] = ('ERROR', '{0}'.format(reason))
-                    
+
                     elif status == 'FOUND':
                         dr[filename] = ('FOUND', '{0}'.format(reason))
 
@@ -396,7 +401,65 @@ class _ClamdGeneric(object):
 
 
 
+    def allmatchscan(self, file):
+        """
+        Scan a file or directory given by filename and after finding a virus within a file, continues scanning for additional viruses.
+        Scan with archive support enabled.
 
+        file (string) : filename or directoy (MUST BE ABSOLUTE PATH !)
+
+        return either :
+          - (dict): {filename1: [(FOUND', 'virusname1'), (FOUND', 'virusname2')], filename2:  [(FOUND', 'virusname1'), (FOUND', 'virusname3')]}
+          - None: if no virus found
+
+        May raise :
+          - ConnectionError: in case of communication problem
+          - socket.timeout: if timeout has expired
+        """
+        assert isstr(file), 'Wrong type for [file], should be a string [was {0}]'.format(type(file))
+
+        dr={}
+
+        if os.path.isdir(file):
+            for path, subdirs, files in os.walk(file):
+                for name in files:
+                     single_file_result = self.allmatchscan(os.path.join(path,name))
+                     if single_file_result:
+                         dr.update(single_file_result)
+
+        else:
+            try:
+                self._init_socket()
+                self._send_command('ALLMATCHSCAN {0}'.format(file))
+            except socket.error:
+                raise ConnectionError('Unable to scan {0}'.format(file))
+
+            result='...'
+
+            while result:
+                try:
+                    result = self._recv_response()
+                except socket.error:
+                    raise ConnectionError('Unable to scan {0}'.format(file))
+
+                if len(result) > 0:
+                    for resline in result.splitlines():
+                        filename, reason, status = self._parse_response(resline)
+
+                        if status == 'ERROR':
+                            if filename not in dr:
+                                dr[filename] = []
+                            dr[filename].append(('ERROR', '{0}'.format(reason)))
+
+                        elif status == 'FOUND':
+                            if filename not in dr:
+                                dr[filename] = []
+                            dr[filename].append(('FOUND', '{0}'.format(reason)))
+            self._close_socket()
+
+        if not dr:
+            return None
+        return dr
 
 
 
@@ -434,10 +497,10 @@ class _ClamdGeneric(object):
             if len(result) > 0:
                 for resline in result.splitlines():
                     filename, reason, status = self._parse_response(resline)
-                    
+
                     if status == 'ERROR':
                         dr[filename] = ('ERROR', '{0}'.format(reason))
-                    
+
                     elif status == 'FOUND':
                         dr[filename] = ('FOUND', '{0}'.format(reason))
 
@@ -448,14 +511,14 @@ class _ClamdGeneric(object):
 
 
 
-    def scan_stream(self, buffer_to_test):
+    def scan_stream(self, stream, chunk_size=4096):
         """
         Scan a buffer
 
         on Python2.X :
-          - buffer_to_test (string): buffer to scan
+          - input (string): buffer to scan
         on Python3.X :
-          - buffer_to_test (bytes or bytearray): buffer to scan
+          - input (bytes or bytearray): buffer to scan
 
         return either:
           - (dict): {filename1: "virusname"}
@@ -467,10 +530,12 @@ class _ClamdGeneric(object):
         """
         if sys.version_info[0] <= 2:
             # Python2
-            assert isstr(buffer_to_test), 'Wrong type fom [buffer_to_test], should be str [was {0}]'.format(type(buffer_to_test))
+            assert hasattr(stream, "read") or isinstance(stream, str), 'Wrong type for [stream], should be str/file-like [was {0}]'.format(type(stream))
         else:
             # Python3
-            assert isinstance(buffer_to_test, bytes) or isinstance(buffer_to_test, bytearray), 'Wrong type fom [buffer_to_test], should be bytes or bytearray [was {0}]'.format(type(buffer_to_test))
+            assert hasattr(stream, "read") or isinstance(stream, (bytes, bytearray)), 'Wrong type for [stream], should be bytes/bytearray/file-like [was {0}]'.format(type(stream))
+
+        is_file_like = hasattr(stream, 'read')
 
         try:
             self._init_socket()
@@ -479,21 +544,33 @@ class _ClamdGeneric(object):
         except socket.error:
             raise ConnectionError('Unable to scan stream')
 
-        # MUST be < StreamMaxLength in /etc/clamav/clamd.conf
-        # or /etc/clamd.conf
-        max_chunk_size = 1024
+        if is_file_like:
+            while True:
+                chunk = stream.read(chunk_size)
+                if not chunk:
+                    break
+                size = struct.pack('!L', len(chunk))
+                try:
+                    self.clamd_socket.send(size)
+                    self.clamd_socket.send(chunk)
+                except socket.error:
+                    raise
 
-        for n in range(1 + int(len(buffer_to_test)/max_chunk_size)):
-            chunk = buffer_to_test[n*max_chunk_size:(n+1)*max_chunk_size]
-            size = struct.pack('!L', len(chunk))
-            try:
-                self.clamd_socket.send(size)
-                self.clamd_socket.send(chunk)
-            except socket.error:
-                break
-        else:
             # Terminating stream
             self.clamd_socket.send(struct.pack('!L', 0))
+        else:
+            # bytearray
+            for n in range(1 + int(len(stream)/chunk_size)):
+                chunk = stream[n*chunk_size:(n+1)*chunk_size]
+                size = struct.pack('!L', len(chunk))
+                try:
+                    self.clamd_socket.send(size)
+                    self.clamd_socket.send(chunk)
+                except socket.error:
+                    raise
+            else:
+                # Terminating stream
+                self.clamd_socket.send(struct.pack('!L', 0))
 
         result='...'
         dr = {}
@@ -522,10 +599,6 @@ class _ClamdGeneric(object):
         return dr
 
 
-
-
-
-    
     def _send_command(self, cmd):
         """
         `man clamd` recommends to prefix commands with z, but we will use \n
@@ -543,7 +616,7 @@ class _ClamdGeneric(object):
         receive response from clamd and strip all whitespace characters
         """
         # If we connect too quickly
-        # sometimes we get a connexion error
+        # sometimes we get a connection error
         # so we retry
         failed_count = 5
         while True:
@@ -585,7 +658,7 @@ class _ClamdGeneric(object):
         """
         self.clamd_socket.close()
         return
-    
+
 
     def _parse_response(self, msg):
         """
@@ -598,7 +671,7 @@ class _ClamdGeneric(object):
             result = left
         else:
             result = ": ".join(left)
-            
+
         if result != 'OK':
             parts = result.split()
             reason = ' '.join(parts[:-1])
@@ -622,7 +695,7 @@ class ClamdUnixSocket(_ClamdGeneric):
     def __init__(self, filename=None, timeout=None):
         """
         Unix Socket Class initialisation
-        
+
         filename (string) : unix socket filename or None to get the socket from /etc/clamav/clamd.conf or /etc/clamd.conf
         timeout (float or None) : socket timeout
         """
@@ -634,7 +707,7 @@ class ClamdUnixSocket(_ClamdGeneric):
                     break
             else:
                 raise ConnectionError('Could not find clamd unix socket from /etc/clamav/clamd.conf or /etc/clamd.conf')
-            
+
             with open(clamdpath, 'r') as conffile:
                 for line in conffile.readlines():
                     try:
@@ -643,15 +716,15 @@ class ClamdUnixSocket(_ClamdGeneric):
                             break
                     except IndexError:
                         pass
-                            
+
                 else:
                     raise ConnectionError('Could not find clamd unix socket from /etc/clamav/clamd.conf or /etc/clamd.conf')
-        
+
         assert isstr(filename), 'Wrong type for [file], should be a string [was {0}]'.format(type(file))
         assert isinstance(timeout, (float, int)) or timeout is None, 'Wrong type for [timeout], should be either None or a float [was {0}]'.format(type(timeout))
 
         _ClamdGeneric.__init__(self)
-        
+
         self.unix_socket = filename
         self.timeout = timeout
 
@@ -675,7 +748,7 @@ class ClamdUnixSocket(_ClamdGeneric):
         except socket.error:
             raise ConnectionError('Could not reach clamd using unix socket ({0})'.format((self.unix_socket)))
         return
-    
+
 
 ############################################################################
 
@@ -691,13 +764,13 @@ class ClamdNetworkSocket(_ClamdGeneric):
         port (int) : TCP port
         timeout (float or None) : socket timeout
         """
-            
+
         assert isinstance(host, str), 'Wrong type for [host], should be a string [was {0}]'.format(type(host))
         assert isinstance(port, int), 'Wrong type for [port], should be an int [was {0}]'.format(type(port))
         assert isinstance(timeout, (float, int)) or timeout is None, 'Wrong type for [timeout], should be either None or a float [was {0}]'.format(type(timeout))
-        
+
         _ClamdGeneric.__init__(self)
-        
+
         self.host = host
         self.port = port
         self.timeout = timeout
@@ -723,7 +796,7 @@ class ClamdNetworkSocket(_ClamdGeneric):
 
         return
 
-    
+
 
 ############################################################################
 
@@ -800,7 +873,7 @@ def _non_regression_test():
 	import doctest
 	doctest.testmod()
 	return
-	
+
 
 ############################################################################
 
